@@ -4,15 +4,14 @@
 #include "beastv2_header.h"
 static void DSVT(BEAST2_BASIS_PTR basis,I32 N,BEAST2_RNDSTREAM* PRAND)
 {
+	I32 rndOrder=ceil((basis->prior.maxOrder+basis->prior.minOrder)/2.0); 
 	I32 maxKnotNum=basis->prior.maxKnotNum;
 	I32 minKnotNum=basis->prior.minKnotNum;
-	I32 rndOrder=ceil((basis->prior.maxOrder+basis->prior.minOrder)/2.0);
-	basis->nKnot=ceil((minKnotNum+maxKnotNum)/2.0);   
-	basis->nKnot=0;                                       
+	basis->nKnot=minKnotNum;  
 	I32 SEP=N/(basis->nKnot+1);
 	I32 initKnot=1+SEP;
-	for (I32 i=1; i <=basis->nKnot; i++) {
-		basis->ORDER[i - 1]=rndOrder;                     
+	for (I32 i=1; i <=basis->nKnot;++i) {
+		basis->ORDER[i - 1]=rndOrder;       
 		basis->KNOT[i - 1]=initKnot;
 		initKnot+=SEP;
 	}
